@@ -63,6 +63,10 @@
 #define V4L2_PIX_FMT_YVYU v4l2_fourcc('Y', 'V', 'Y', 'U')
 #endif
 
+#ifndef V4L2_PIX_FMT_SN9C20X_I420
+#define V4L2_PIX_FMT_SN9C20X_I420  v4l2_fourcc('S', '9', '2', '0')
+#endif
+
 #define V4LCONVERT_ERROR_MSG_SIZE 256
 #define V4LCONVERT_MAX_FRAMESIZES 16
 
@@ -96,7 +100,11 @@ struct v4lconvert_data {
 };
 
 struct v4lconvert_flags_info {
-  const char *card;
+  unsigned short vendor_id;
+  unsigned short product_id;
+/* We could also use the USB manufacturer and product strings some devices have
+  const char *manufacturer;
+  const char *product; */
   int flags;
 };
 
@@ -129,7 +137,13 @@ void v4lconvert_yvyu_to_rgb24(const unsigned char *src, unsigned char *dst,
 void v4lconvert_yvyu_to_bgr24(const unsigned char *src, unsigned char *dst,
   int width, int height);
 
-void v4lconvert_yvyu_to_yuv420(const unsigned char *src, unsigned char *dst,
+void v4lconvert_uyvy_to_rgb24(const unsigned char *src, unsigned char *dst,
+  int width, int height);
+
+void v4lconvert_uyvy_to_bgr24(const unsigned char *src, unsigned char *dst,
+  int width, int height);
+
+void v4lconvert_uyvy_to_yuv420(const unsigned char *src, unsigned char *dst,
   int width, int height, int yvu);
 
 void v4lconvert_swap_rgb(const unsigned char *src, unsigned char *dst,
@@ -147,6 +161,9 @@ void v4lconvert_spca505_to_yuv420(const unsigned char *src, unsigned char *dst,
 void v4lconvert_spca508_to_yuv420(const unsigned char *src, unsigned char *dst,
   int width, int height, int yvu);
 
+void v4lconvert_sn9c20x_to_yuv420(const unsigned char *src, unsigned char *dst,
+  int width, int height, int yvu);
+
 void v4lconvert_decode_spca561(const unsigned char *src, unsigned char *dst,
   int width, int height);
 
