@@ -21,6 +21,8 @@
 #include <stdlib.h>
 #include <syscall.h>
 #include <unistd.h>
+#include <sys/stat.h>
+#include <sys/types.h>
 #include "libv4lconvert.h"
 #include "libv4lconvert-priv.h"
 
@@ -44,6 +46,8 @@ static const struct v4lconvert_pixfmt supported_src_pixfmts[] = {
   SUPPORTED_DST_PIXFMTS,
   { V4L2_PIX_FMT_YUYV,    0 },
   { V4L2_PIX_FMT_YVYU,    0 },
+  { V4L2_PIX_FMT_UYVY,    0 },
+  { V4L2_PIX_FMT_SN9C20X_I420, 0 },
   { V4L2_PIX_FMT_SBGGR8,  0 },
   { V4L2_PIX_FMT_SGBRG8,  0 },
   { V4L2_PIX_FMT_SGRBG8,  0 },
@@ -65,11 +69,10 @@ static const struct v4lconvert_pixfmt supported_dst_pixfmts[] = {
 
 /* List of cams which need special flags */
 static const struct v4lconvert_flags_info v4lconvert_flags[] = {
-  { "SPC 200NC      ", V4LCONVERT_ROTATE_180 },
-  { "SPC 300NC      ", V4LCONVERT_ROTATE_180 }, /* Unconfirmed ! */
-  { "SPC210NC       ", V4LCONVERT_ROTATE_180 },
-  { "USB Camera (0471:0326)", V4LCONVERT_ROTATE_180 }, /* SPC300NC */
-  { "USB Camera (093a:2476)", V4LCONVERT_ROTATE_180 }, /* Genius E-M 112 */
+  { 0x0471, 0x0325, V4LCONVERT_ROTATE_180 }, /* Philips SPC200NC */
+  { 0x0471, 0x0326, V4LCONVERT_ROTATE_180 }, /* Philips SPC300NC */
+  { 0x0471, 0x032d, V4LCONVERT_ROTATE_180 }, /* Philips SPC210NC */
+  { 0x093a, 0x2476, V4LCONVERT_ROTATE_180 }, /* Genius E-M 112 */
 };
 
 /* List of well known resolutions which we can get by cropping somewhat larger
@@ -85,6 +88,46 @@ static const int v4lconvert_crop_res[][2] = {
   { 176, 144 },
 };
 
+static int v4lconvert_get_flags(int fd)
+{
+  struct stat st;
+  FILE *f;
+  char sysfs_name[512];
+  unsigned short vendor_id = 0;
+  unsigned short product_id = 0;
+  int i;
+
+  if (fstat(fd, &st) || !S_ISCHR(st.st_mode))
+    return 0; /* Should never happen */
+
+  /* Get product ID */
+  snprintf(sysfs_name, sizeof(sysfs_name),
+	   "/sys/class/video4linux/video%d/device/idVendor",
+	   minor(st.st_rdev));
+  f = fopen(sysfs_name, "r");
+  if (!f)
+    return 0; /* Not an USB device (or no sysfs) */
+  i = fscanf(f, "%hx", &vendor_id);
+  fclose(f);
+
+  /* Get product ID */
+  snprintf(sysfs_name, sizeof(sysfs_name),
+	   "/sys/class/video4linux/video%d/device/idProduct",
+	   minor(st.st_rdev));
+  f = fopen(sysfs_name, "r");
+  if (!f)
+    return 0; /* Should never happen */
+  i = fscanf(f, "%hx", &product_id);
+  fclose(f);
+
+  for (i = 0; i < ARRAY_SIZE(v4lconvert_flags); i++)
+    if (v4lconvert_flags[i].vendor_id == vendor_id &&
+	v4lconvert_flags[i].product_id == product_id)
+      return v4lconvert_flags[i].flags;
+
+  return 0;
+}
+
 struct v4lconvert_data *v4lconvert_create(int fd)
 {
   int i, j;
@@ -116,12 +159,8 @@ struct v4lconvert_data *v4lconvert_create(int fd)
   data->no_formats = i;
 
   /* Check if this cam has any special flags */
+  data->flags = v4lconvert_get_flags(data->fd);
   if (syscall(SYS_ioctl, fd, VIDIOC_QUERYCAP, &cap) == 0) {
-    for (i = 0; i < ARRAY_SIZE(v4lconvert_flags); i++)
-      if (!strcmp((const char *)v4lconvert_flags[i].card, (char *)cap.card)) {
-	data->flags = v4lconvert_flags[i].flags;
-	break;
-      }
     if (!strcmp((char *)cap.driver, "uvcvideo"))
       data->flags |= V4LCONVERT_IS_UVC;
   }
@@ -211,7 +250,7 @@ static int v4lconvert_do_try_format_uvc(struct v4lconvert_data *data,
 			       size_y_diff * size_y_diff;
 
       if (size_diff < closest_fmt_size_diff) {
-        closest_fmt_size_diff = size_diff;
+	closest_fmt_size_diff = size_diff;
 	best_framesize = i;
       }
     }
@@ -514,30 +553,11 @@ static int v4lconvert_convert_pixfmt(struct v4lconvert_data *data,
       }
       break;
 
-    case V4L2_PIX_FMT_SBGGR8:
-    case V4L2_PIX_FMT_SGBRG8:
-    case V4L2_PIX_FMT_SGRBG8:
-    case V4L2_PIX_FMT_SRGGB8:
-      switch (dest_pix_fmt) {
-      case V4L2_PIX_FMT_RGB24:
-	v4lconvert_bayer_to_rgb24(src, dest, width, height, src_pix_fmt);
-	break;
-      case V4L2_PIX_FMT_BGR24:
-	v4lconvert_bayer_to_bgr24(src, dest, width, height, src_pix_fmt);
-	break;
-      case V4L2_PIX_FMT_YUV420:
-	v4lconvert_bayer_to_yuv420(src, dest, width, height, src_pix_fmt, 0);
-	break;
-      case V4L2_PIX_FMT_YVU420:
-	v4lconvert_bayer_to_yuv420(src, dest, width, height, src_pix_fmt, 1);
-	break;
-      }
-      break;
-
-    /* YUYV line by line formats */
+    /* Custom cam specific YUV formats */
     case V4L2_PIX_FMT_SPCA501:
     case V4L2_PIX_FMT_SPCA505:
     case V4L2_PIX_FMT_SPCA508:
+    case V4L2_PIX_FMT_SN9C20X_I420:
     {
       unsigned char *d;
       int yvu = 0;
@@ -564,6 +584,9 @@ static int v4lconvert_convert_pixfmt(struct v4lconvert_data *data,
 	case V4L2_PIX_FMT_SPCA508:
 	  v4lconvert_spca508_to_yuv420(src, d, width, height, yvu);
 	  break;
+	case V4L2_PIX_FMT_SN9C20X_I420:
+	  v4lconvert_sn9c20x_to_yuv420(src, d, width, height, yvu);
+	  break;
       }
 
       switch (dest_pix_fmt) {
@@ -585,7 +608,6 @@ static int v4lconvert_convert_pixfmt(struct v4lconvert_data *data,
     case V4L2_PIX_FMT_PAC207:
     {
       unsigned char *tmpbuf;
-      unsigned int bayer_fmt = 0;
 
       tmpbuf = v4lconvert_alloc_buffer(data, width * height,
 	    &data->convert_pixfmt_buf, &data->convert_pixfmt_buf_size);
@@ -595,34 +617,41 @@ static int v4lconvert_convert_pixfmt(struct v4lconvert_data *data,
       switch (src_pix_fmt) {
 	case V4L2_PIX_FMT_SPCA561:
 	  v4lconvert_decode_spca561(src, tmpbuf, width, height);
-	  bayer_fmt = V4L2_PIX_FMT_SGBRG8;
+	  src_pix_fmt = V4L2_PIX_FMT_SGBRG8;
 	  break;
 	case V4L2_PIX_FMT_SN9C10X:
 	  v4lconvert_decode_sn9c10x(src, tmpbuf, width, height);
-	  bayer_fmt = V4L2_PIX_FMT_SBGGR8;
+	  src_pix_fmt = V4L2_PIX_FMT_SBGGR8;
 	  break;
 	case V4L2_PIX_FMT_PAC207:
 	  v4lconvert_decode_pac207(src, tmpbuf, width, height);
-	  bayer_fmt = V4L2_PIX_FMT_SBGGR8;
+	  src_pix_fmt = V4L2_PIX_FMT_SBGGR8;
 	  break;
       }
+      src = tmpbuf;
+      /* Deliberate fall through to raw bayer fmt code! */
+    }
 
+    /* Raw bayer formats */
+    case V4L2_PIX_FMT_SBGGR8:
+    case V4L2_PIX_FMT_SGBRG8:
+    case V4L2_PIX_FMT_SGRBG8:
+    case V4L2_PIX_FMT_SRGGB8:
       switch (dest_pix_fmt) {
       case V4L2_PIX_FMT_RGB24:
-	v4lconvert_bayer_to_rgb24(tmpbuf, dest, width, height, bayer_fmt);
+	v4lconvert_bayer_to_rgb24(src, dest, width, height, src_pix_fmt);
 	break;
       case V4L2_PIX_FMT_BGR24:
-	v4lconvert_bayer_to_bgr24(tmpbuf, dest, width, height, bayer_fmt);
+	v4lconvert_bayer_to_bgr24(src, dest, width, height, src_pix_fmt);
 	break;
       case V4L2_PIX_FMT_YUV420:
-	v4lconvert_bayer_to_yuv420(tmpbuf, dest, width, height, bayer_fmt, 0);
+	v4lconvert_bayer_to_yuv420(src, dest, width, height, src_pix_fmt, 0);
 	break;
       case V4L2_PIX_FMT_YVU420:
-	v4lconvert_bayer_to_yuv420(tmpbuf, dest, width, height, bayer_fmt, 1);
+	v4lconvert_bayer_to_yuv420(src, dest, width, height, src_pix_fmt, 1);
 	break;
       }
       break;
-    }
 
     case V4L2_PIX_FMT_RGB24:
       switch (dest_pix_fmt) {
@@ -710,10 +739,29 @@ static int v4lconvert_convert_pixfmt(struct v4lconvert_data *data,
 	v4lconvert_yvyu_to_bgr24(src, dest, width, height);
 	break;
       case V4L2_PIX_FMT_YUV420:
-	v4lconvert_yvyu_to_yuv420(src, dest, width, height, 0);
+	/* Note we use yuyv_to_yuv420 not v4lconvert_yvyu_to_yuv420,
+	   with the last argument reversed to make it have as we want */
+	v4lconvert_yuyv_to_yuv420(src, dest, width, height, 1);
 	break;
       case V4L2_PIX_FMT_YVU420:
-	v4lconvert_yvyu_to_yuv420(src, dest, width, height, 1);
+	v4lconvert_yuyv_to_yuv420(src, dest, width, height, 0);
+	break;
+      }
+      break;
+
+    case V4L2_PIX_FMT_UYVY:
+      switch (dest_pix_fmt) {
+      case V4L2_PIX_FMT_RGB24:
+	v4lconvert_uyvy_to_rgb24(src, dest, width, height);
+	break;
+      case V4L2_PIX_FMT_BGR24:
+	v4lconvert_uyvy_to_bgr24(src, dest, width, height);
+	break;
+      case V4L2_PIX_FMT_YUV420:
+	v4lconvert_uyvy_to_yuv420(src, dest, width, height, 0);
+	break;
+      case V4L2_PIX_FMT_YVU420:
+	v4lconvert_uyvy_to_yuv420(src, dest, width, height, 1);
 	break;
       }
       break;
@@ -952,7 +1000,7 @@ int v4lconvert_enum_frameintervals(struct v4lconvert_data *data,
     int frmival_pixformat = frmival->pixel_format;
     int dest_pixformat = dest_fmt.fmt.pix.pixelformat;
     V4LCONVERT_ERR("Could not find matching framesize for: %c%c%c%c %dx%d "
-            "closest match: %c%c%c%c %dx%d\n",
+	    "closest match: %c%c%c%c %dx%d\n",
 	    frmival_pixformat & 0xff,
 	    (frmival_pixformat >> 8) & 0xff,
 	    (frmival_pixformat >> 16) & 0xff,
@@ -976,8 +1024,8 @@ int v4lconvert_enum_frameintervals(struct v4lconvert_data *data,
     int dest_pixfmt = dest_fmt.fmt.pix.pixelformat;
     int src_pixfmt  = src_fmt.fmt.pix.pixelformat;
     V4LCONVERT_ERR("Could not enum frameival index: %d for: %c%c%c%c %dx%d "
-            "using src: %c%c%c%c %dx%d, error: %s\n",
-            frmival->index,
+	    "using src: %c%c%c%c %dx%d, error: %s\n",
+	    frmival->index,
 	    dest_pixfmt & 0xff,
 	    (dest_pixfmt >> 8) & 0xff,
 	    (dest_pixfmt >> 16) & 0xff,
